#
# Create arbitrarily meshed sphere geometry
#
#  Created by Adam Billyard on 08/09/2010.
#
# usage:  Primitives::sphere(radius, subdivision)   #  3 or 4 gives a good result.
 
module Primitives

def Primitives::subdiv(m, depth)

	if depth > 0
		mprime = []
		m.each { |t|
			t01 = Geom::linear_combination(0.5, t[0], 0.5, t[1])
			t12 = Geom::linear_combination(0.5, t[1], 0.5, t[2])
			t20 = Geom::linear_combination(0.5, t[2], 0.5, t[0])
		
			mprime += subdiv([[t[0],t01,t20], [t[1],t12,t01], [t[2],t20,t12], [t01,t12,t20]], depth-1)
		}
	else
		mprime = m	
	end

	return mprime
end

def Primitives::pscale(p, r)
	s = r / Math::sqrt(p.x * p.x + p.y * p.y + p.z * p.z)

	Geom::Point3d.new(p.x * s, p.y * s, p.z * s)
end

def Primitives::sphere(r, depth, ltm = Geom::Transformation.new)

	tri0 = [Geom::Point3d.new(-1,-1, 1), Geom::Point3d.new(1, -1, 1), Geom::Point3d.new( 1, 1, 1)]
	tri1 = [Geom::Point3d.new(-1,-1, 1), Geom::Point3d.new(1,  1, 1), Geom::Point3d.new(-1, 1, 1)]
	o = Geom::Point3d.new
	xforms = [Geom::Transformation.axes(o, X_AXIS,Y_AXIS,Z_AXIS),
			Geom::Transformation.axes(o, X_AXIS,Y_AXIS,Z_AXIS) * Geom::Transformation.rotation(o, X_AXIS, Math::PI * 0.5),
			Geom::Transformation.axes(o, X_AXIS,Y_AXIS,Z_AXIS) * Geom::Transformation.rotation(o, X_AXIS, Math::PI * 1.0),
			Geom::Transformation.axes(o, X_AXIS,Y_AXIS,Z_AXIS) * Geom::Transformation.rotation(o, X_AXIS, Math::PI * -0.5),
			Geom::Transformation.axes(o, X_AXIS,Y_AXIS,Z_AXIS) * Geom::Transformation.rotation(o, Y_AXIS, Math::PI * 0.5),
			Geom::Transformation.axes(o, X_AXIS,Y_AXIS,Z_AXIS) * Geom::Transformation.rotation(o, Y_AXIS, Math::PI * -0.5)
			 ]
	m = []
	for omat in xforms
		mat = omat
		m += subdiv([ [tri0[0].transform(mat), tri0[1].transform(mat), tri0[2].transform(mat)] ], depth)
		m += subdiv([ [tri1[0].transform(mat), tri1[1].transform(mat), tri1[2].transform(mat)] ], depth)
	end
	m.each {|t|
		Sketchup.active_model.entities.add_face [pscale(t[0],r), pscale(t[1],r),pscale(t[2],r)]
	}
	return m.length
end

end
